import requests
import json


class WasdLib:
    def __init__(self, stream_url):
        self.channel_id = self.get_channel_id_from_url(stream_url)
        self.stream_name = ""
        self.stream_status = "STOPPED"

        wasd_session = requests.session()
        wasd_session.get("https://wasd.tv/api/auth/anon-token")

    def get_json_of_stream_by_offset(self, limit=1, offset=0, status='INIT,IDLE,RUNNING,STOPPED'):
        payload = {
            'limit': limit,
            'offset': offset,
            'channel_id': self.channel_id,
            'media_container_type': 'SINGLE,COOP',
            'media_container_status': status,
            'order_direction': 'DESC'
        }

        api_url = "https://wasd.tv/api/v2/media-containers"
        result_json = requests.get(api_url, params=payload)
        result = json.loads(result_json.content)
        return result

    def get_stream(self, selected_stream=0) -> dict:
        """ Get playlist m3u8 for current stream """
        stream_json = self.get_json_of_stream_by_offset(1, selected_stream)

        tags = []
        for tag in stream_json['result'][0]['tags']:
            tags.append(tag['tag_name'])

        result = {
            'stream_name': stream_json['result'][0]['media_container_name'],
            'stream_status': stream_json['result'][0]['media_container_status'],
            'stream_m3u8':
                stream_json['result'][0]['media_container_streams'][0]['stream_media'][0]['media_meta']['media_url'],
            'archive_stream_m3u8':
                stream_json['result'][0]['media_container_streams'][0]['stream_media'][0]['media_meta'][
                    'media_archive_url'],
            'stream_tags':
                tags
        }

        self.stream_name = result['stream_name']
        self.stream_status = result['stream_name']
        return result

    def get_streams(self, page=1) -> tuple:
        """ Get tuple of streams
        Bugs: wasd.tv returns only up to 30 streams, regardless of the limit and offset. Also, if not point stream status, return only up to 15
        """

        limit = 10
        offset = (page * 10) - 10
        json_r = self.get_json_of_stream_by_offset(limit, offset)

        list_of_streams = []
        for i in json_r['result']:
            name = i['media_container_name']
            list_of_streams.append(name)
        return tuple(list_of_streams)

    @staticmethod
    def get_channel_id_from_url(stream_url) -> int:
        """ Convert streamer nickname to channel id """
        nickname = stream_url.split("/")[-1:][0]
        r = requests.get(f"https://wasd.tv/api/channels/nicknames/{nickname}")
        json_r = json.loads(r.content)
        channel_id = int(json_r['result']['channel_id'])
        return channel_id



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='wasd2play',
    version='0.4.6',
    packages=['wasd2play'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=["requests>=2.24.0"],
    url='https://github.com/viktor02/wasd2play',
    license='MIT',
    author='Viktor Karpov',
    author_email='v@vitka-k.ru',
    description='Open wasd.tv streams in your favorite player!',
    entry_points={
        'console_scripts': [
            'wasd2play=wasd2play:main',
        ],
    },
    long_description=long_description,
    long_description_content_type="text/markdown",
)



from wasd2play.wasd_lib import WasdLib
import subprocess
import argparse


def open_player(url, player, selected_stream=0, force_open_last_stream=False,
                download_stream=False, download_stream_nvenc=False, debug_info=False):
    """ Open player with stream """
    try:
        wasd = WasdLib(url)

        print("[wasd2play] Got a stream playlist...")
        stream_info = wasd.get_stream(selected_stream)

        # Set to current url
        stream_url = stream_info['stream_m3u8']

        # If stream offline, open last
        if stream_info['stream_status'] == "STOPPED" or force_open_last_stream:
            print("[wasd2play] Open last archived stream...")
            stream_url = stream_info['archive_stream_m3u8']
        else:
            print("[wasd2play] Open current stream...")

        print(f"[wasd2play] {wasd.stream_name} in your {player} player!")

        tags = ", ".join(stream_info['stream_tags'])
        print(f"[wasd2play] Tags: {tags}")

        if debug_info:
            print(f"URL: {stream_url}\n\n{stream_info}")
    except KeyError:
        print("[wasd2play] Wrong url/streamer nickname. Stopping.")
        return 0
    except OSError:
        print("[wasd2play] OSError. Stopping.")
        return 0

    try:
        if download_stream:
            subprocess.run(f"ffmpeg -i {stream_url} -c:v libx264 -b:v 5M -maxrate 5M -bufsize 8M -c:a copy \"{wasd.stream_name}\".mkv")
        elif download_stream_nvenc:
            subprocess.run(
                f"ffmpeg -hwaccel cuvid -i {stream_url} -c:v hevc_nvenc -b:v 5M -maxrate 7M -bufsize 8M -c:a copy \"{wasd.stream_name}\".mkv")
        else:
            subprocess.run([player, stream_url], stdout=subprocess.PIPE, check=True)
    except FileNotFoundError:
        print("[wasd2play] Error: player or ffmpeg not found. Try use another player with -p flag. Stopping.")
    except KeyboardInterrupt:
        return 0


def show_streams(url, page=1):
    wasd = WasdLib(url)

    print("[wasd2play] Got a streams list...\n")

    streams_list = wasd.get_streams(page)
    count = (page * 10) - 10
    for stream_name in streams_list:
        print(count, stream_name)
        count += 1
    print(f"Page {page}/3")


def runner():
    """ Entry point """

    my_parser = argparse.ArgumentParser(prog='wasd2play',
                                        usage='%(prog)s [options] url',
                                        description='Open any stream in your favorite player!',
                                        epilog='Have a good time!')

    my_parser.add_argument('url',
                           help='stream url')

    # Optional args
    my_parser.add_argument('-p', '--player',
                           help='Point your player',
                           default='vlc',
                           dest='your_player')
    my_parser.add_argument('-l', '--last',
                           action='store_true',
                           dest="last_stream",
                           help='Open last stream')

    # Show and select streams
    my_parser.add_argument('-s', '--select',
                           action='store',
                           dest="selected",
                           type=int,
                           help='Select stream')
    my_parser.add_argument('-ls', '--list',
                           action='store',
                           dest="page",
                           nargs='?',
                           const=1,
                           type=int,
                           help='Show recent streams page by page')

    # Download with ffmpeg
    my_parser.add_argument('-d', '--download',
                           action='store_true',
                           dest='download_stream',
                           help='Download stream with ffmpeg')
    my_parser.add_argument('-dnv', '--download-nvenc',
                           action='store_true',
                           dest='download_stream_nvenc',
                           help='Download stream with ffmpeg using nvenc')
    my_parser.add_argument('--debug',
                           action='store_true',
                           dest='debug_info',
                           help='Show debug info')

    args = my_parser.parse_args()

    welcome_message = "##\n## wasd2play\n##\n"
    print(welcome_message)

    if (args.page is not None) and (args.page >= 1):
        show_streams(args.url, args.page)
        return 0

    open_player(args.url, args.your_player, args.selected,
                args.last_stream, args.download_stream, args.download_stream_nvenc, args.debug_info)
